#include <glut.h>
#include <math.h>
#include "glwindow.h"
#include "glfile.h"

#define FOVY            40.0
#define MINVIEWDISTANCE 1.0
#define MAXVIEWDISTANCE 1000.0

GLfloat xRoll = 0.0;
GLfloat yRoll = 0.0;
GLfloat zRoll = 0.0;
GLfloat amount = 0.0;
GLfloat zoom = -250.0;
/* initialize rotation to identity */
GLfloat rotationMatrix[16] = { 1, 0, 0, 0,
                               0, 1, 0, 0,
                               0, 0, 1, 0,
                               0, 0, 0, 1 };

enum state { MOVE_NONE=0, MOVE_ZOOM, MOVE_ROTATE };
GLint mouseState = MOVE_NONE;
GLint selectedMotion = MOVE_ROTATE;
GLint xDownPos = 0.0;
GLint yDownPos = 0.0;
GLint zoomPos = 0.0;
GLint windowWidth, windowHeight;

extern modelData *model;

/* openGL settings */
void setWindowSize( int width, int height )
{
   /* setup the 3D perspective window */
   glMatrixMode( GL_PROJECTION );
   glLoadIdentity();
   gluPerspective( FOVY, (float)width / (float)height, MINVIEWDISTANCE, MAXVIEWDISTANCE );
   glMatrixMode( GL_MODELVIEW );
   /* make a viewport the acual size of the window */
   glViewport( 0, 0, width, height );
   /* store for later */
   windowWidth = width;
   windowHeight = height;
}

void mouseFunc( int button, int state, int x, int y )
{
   switch( button )
   {
      case GLUT_LEFT_BUTTON:
         switch( state )
         {
            case GLUT_DOWN:
               /* store the down spot for later */
               xDownPos = x;
               yDownPos = y;
               zoomPos = y;
               mouseState = selectedMotion;
               break;
            case GLUT_UP:
               mouseState = MOVE_NONE;
               updateRotation();
               amount = 0;
               break;
         }
         break;
   }
}

void mouseMotionFunc( int x, int y )
{
   /* find out what action to perform */
   switch( mouseState )
   {
      case MOVE_ZOOM:
         /* change zoom and reset reference */
         zoom += ( y - zoomPos );
         zoomPos = y;
         break;
      case MOVE_ROTATE:
         {
            /* calculate percentages from centre of screen */
            float horPct = (float)(x - windowWidth/2 ) / (float)(windowWidth/2);
            float verPct = (float)(y - windowHeight/2 ) / (float)(windowHeight/2);

            /* get percentages for rotations */
            yRoll =  (1.0 - fabs(verPct))*(x - xDownPos);
            xRoll = (1.0 - fabs(horPct))*(y - yDownPos);
            zRoll = horPct/2*(yDownPos - y) - verPct/2*(xDownPos - x);

            /* amount is distance between current and initial pos */
            amount = sqrt( (xDownPos-x)*(xDownPos-x) + (yDownPos-y)*(yDownPos-y));
         }
         break;
   }
}

void idleFunc()
{
   /* tell glut that we want our window updated */
   glutPostRedisplay();
}

void paintWindow( )
{
   /* clear the buffer */
   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

   /* no rotations */
   glLoadIdentity();

   /* move the image back from the camera */
   glTranslatef( 0.0, 0.0, zoom );

   /* rotate image about its origin */
   if( amount )
      glRotatef( amount, xRoll, yRoll, zRoll );
   glMultMatrixf( rotationMatrix );

   if( !model )
   {
      /* placeholder object */
      glutWireTorus( 25.0, 50.0, 15, 15 );
   } else {
      int i;


      glBegin( GL_POINTS );
         for( i=0; i<model->numPoints; i++ )
            glVertex3fv( model->pointList[i].pt );
      glEnd();
   }

   /* swap the back draw buffer to the front */
   glutSwapBuffers( );
}

void updateRotation()
{
   /* save where we are */
   glPushMatrix();
   /* update our stored matrix with a new rotation */
   glLoadIdentity();
   if( amount )
      glRotatef( amount, xRoll, yRoll, zRoll );
   glMultMatrixf( rotationMatrix );
   glGetFloatv( GL_MODELVIEW_MATRIX, rotationMatrix );
   glPopMatrix();
}

void menuFuncMain( int id )
{
}

void menuFuncDisplay( int id )
{
}

void menuFuncLoad( int id )
{
}

void menuFuncMotionSelect( int id )
{
   /* find out which motion type was selected */
   switch( id )
   {
      case MENU_MOTION_ROTATE:
         selectedMotion = MOVE_ROTATE;
         break;
      case MENU_MOTION_ZOOM:
         selectedMotion = MOVE_ZOOM;
         break;
   }
}
